package org.eclipse.aether.graph;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *  http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import java.util.List;

/**
 * A filter to include/exclude dependency nodes during other operations.
 */
public interface DependencyFilter
{

    /**
     * Indicates whether the specified dependency node shall be included or excluded.
     * 
     * @param node The dependency node to filter, must not be {@code null}.
     * @param parents The (read-only) chain of parent nodes that leads to the node to be filtered, must not be
     *            {@code null}. Iterating this (possibly empty) list walks up the dependency graph towards the root
     *            node, i.e. the immediate parent node (if any) is the first node in the list. The size of the list also
     *            denotes the zero-based depth of the filtered node.
     * @return {@code true} to include the dependency node, {@code false} to exclude it.
     */
    boolean accept( DependencyNode node, List<DependencyNode> parents );

}
